@@ -16,6 +16,24 @@ from .. import tsutils
 
 warnings.filterwarnings("ignore")
 
+ldocstrings = tsutils.docstrings
+ldocstrings[
+    "xydata"
+] = """If the input 'x,y' dataset(s) is organized as
+            'index,x1,y1,x2,y2,x3,y3,...,xN,yN' then the 'index' is ignored.
+            If there is one 'x,y' dataset then it can be organized as 'index,y'
+            where 'index' is used for 'x'.  The "columns" keyword can be used
+            to duplicate or change the order of all the data columns."""
+ldocstrings[
+    "ydata"
+] = """Data must be organized as 'index,y1,y2,y3,...,yN'.  The 'index' is
+            ignored and all data columns are plotted.  The "columns" keyword
+            can be used to duplicate or change the order of all the data
+            columns."""
+ldocstrings[
+    "yone"
+] = """Data must be organized as 'index,y1'.  Can only plot one series."""
+
 MARKER_LIST = [
     ".",
     ",",
@@ -43,7 +61,9 @@ MARKER_LIST = [
 
 COLOR_LIST = ["b", "g", "r", "c", "m", "y", "k"]
 
-LINE_LIST = ["-", "--", "-.", ":"]
+LINE_LIST = ["-", "--", "-.", ":", "solid", "dashed", "dashdot", "dotted"]
+
+HATCH_LIST = ["/", "\\", "|", "-", "+", "x", "o", "O", ".", "*"]
 
 
 def _know_your_limits(xylimits, axis="arithmetic"):
@@ -66,8 +86,8 @@ def _know_your_limits(xylimits, axis="arithmetic"):
             raise ValueError(
                 tsutils.error_wrapper(
                     """
-Both limits must be between 0 and 1 for the
-'normal', 'lognormal', or 'weibull' axis.
+Both limits must be between 0 and 1 for the 'normal', 'lognormal', or 'weibull'
+axis.
 
 Instead you have {0}.
 """.format(
@@ -137,6 +157,7 @@ def plot_cli(
     colors="auto",
     linestyles="auto",
     markerstyles=" ",
+    bar_hatchstyles="auto",
     style="auto",
     logx=False,
     logy=False,
@@ -191,41 +212,60 @@ def plot_cli(
         Can be one of the following:
 
         time
-            Standard time series plot.  Time is the index, and plots each
-            column of data.
+            Standard time series plot.
+
+            Data must be organized as 'index,y1,y2,y3,...,yN'.  The 'index'
+            must be a date/time and all data columns are plotted.  Legend names
+            are taken from the column names in the first row unless over-ridden
+            by the `legend_names` keyword.
         xy
-            An (x,y) plot, also know as a scatter plot.  Data must be organized
-            as x1,y1,x2,y2,x3,y3,....
+            An 'x,y' plot, also know as a scatter plot.
+
+            {xydata}
         double_mass
-            An (x,y) plot of the cumulative sum of x and y.  Data must be
-            organized as x1,y1,x2,y2,x3,y3,....
+            An 'x,y' plot of the cumulative sum of x and y.
+
+            {xydata}
         boxplot
             Box extends from lower to upper quartile, with line at the
             median.  Depending on the statistics, the wiskers represent
             the range of the data or 1.5 times the inter-quartile range
-            (Q3 - Q1).  Data should be organized as y1,y2,y3,....
+            (Q3 - Q1).
+
+            {ydata}
         scatter_matrix
             Plots all columns against each other in a matrix, with the diagonal
             plots either histogram or KDE probability distribution
             depending on `scatter_matrix_diagonal` keyword.
+
+            {ydata}
         lag_plot
-            Indicates structure in the data.  Only available for a single
-            time-series.
+            Indicates structure in the data.
+
+            {yone}
         autocorrelation
             Plot autocorrelation.  Only available for a single time-series.
+
+            {yone}
         bootstrap
             Visually assess aspects of a data set by plotting random
             selections of values.  Only available for a single time-series.
+
+            {yone}
         histogram
             Calculate and create a histogram plot.  See 'kde' for a smooth
             representation of a histogram.
         kde
             This plot is an estimation of the probability density function
             based on the data called kernel density estimation (KDE).
+
+            {ydata}
         kde_time
             This plot is an estimation of the probability density function
             based on the data called kernel density estimation (KDE) combined
             with a time-series plot.
+
+            {ydata}
         bar
             Column plot.
         barh
@@ -373,23 +413,23 @@ def plot_cli(
         Separated 'colors', 'linestyles', and 'markerstyles' instead of using
         the 'style' keyword.
 
-        +------+--------------+
-        | Code | Lines        |
-        +======+==============+
-        | \-   | solid        |
-        +------+--------------+
-        | --   | dashed       |
-        +------+--------------+
-        | -.   | dash_dot     |
-        +------+--------------+
-        | :    | dotted       |
-        +------+--------------+
-        | None | draw nothing |
-        +------+--------------+
-        | ' '  | draw nothing |
-        +------+--------------+
-        | ''   | draw nothing |
-        +------+--------------+
+        +---------+--------------+
+        | Code    | Lines        |
+        +=========+==============+
+        | ``-``   | solid        |
+        +---------+--------------+
+        | --      | dashed       |
+        +---------+--------------+
+        | -.      | dash_dot     |
+        +---------+--------------+
+        | :       | dotted       |
+        +---------+--------------+
+        | None    | draw nothing |
+        +---------+--------------+
+        | ' '     | draw nothing |
+        +---------+--------------+
+        | ''      | draw nothing |
+        +---------+--------------+
 
         Line reference:
         http://matplotlib.org/api/artist_api.html
@@ -403,57 +443,57 @@ def plot_cli(
         Separated 'colors', 'linestyles', and 'markerstyles' instead of using
         the 'style' keyword.
 
-        +------+----------------+
-        | Code | Markers        |
-        +======+================+
-        | .    | point          |
-        +------+----------------+
-        | o    | circle         |
-        +------+----------------+
-        | v    | triangle down  |
-        +------+----------------+
-        | ^    | triangle up    |
-        +------+----------------+
-        | <    | triangle left  |
-        +------+----------------+
-        | >    | triangle right |
-        +------+----------------+
-        | 1    | tri_down       |
-        +------+----------------+
-        | 2    | tri_up         |
-        +------+----------------+
-        | 3    | tri_left       |
-        +------+----------------+
-        | 4    | tri_right      |
-        +------+----------------+
-        | 8    | octagon        |
-        +------+----------------+
-        | s    | square         |
-        +------+----------------+
-        | p    | pentagon       |
-        +------+----------------+
-        | \*   | star           |
-        +------+----------------+
-        | h    | hexagon1       |
-        +------+----------------+
-        | H    | hexagon2       |
-        +------+----------------+
-        | \+   | plus           |
-        +------+----------------+
-        | x    | x              |
-        +------+----------------+
-        | D    | diamond        |
-        +------+----------------+
-        | d    | thin diamond   |
-        +------+----------------+
-        | _    | hline          |
-        +------+----------------+
-        | None | nothing        |
-        +------+----------------+
-        | ' '  | nothing        |
-        +------+----------------+
-        | ''   | nothing        |
-        +------+----------------+
+        +-------+----------------+
+        | Code  | Markers        |
+        +=======+================+
+        | .     | point          |
+        +-------+----------------+
+        | o     | circle         |
+        +-------+----------------+
+        | v     | triangle down  |
+        +-------+----------------+
+        | ^     | triangle up    |
+        +-------+----------------+
+        | <     | triangle left  |
+        +-------+----------------+
+        | >     | triangle right |
+        +-------+----------------+
+        | 1     | tri_down       |
+        +-------+----------------+
+        | 2     | tri_up         |
+        +-------+----------------+
+        | 3     | tri_left       |
+        +-------+----------------+
+        | 4     | tri_right      |
+        +-------+----------------+
+        | 8     | octagon        |
+        +-------+----------------+
+        | s     | square         |
+        +-------+----------------+
+        | p     | pentagon       |
+        +-------+----------------+
+        | ``*`` | star           |
+        +-------+----------------+
+        | h     | hexagon1       |
+        +-------+----------------+
+        | H     | hexagon2       |
+        +-------+----------------+
+        | ``+`` | plus           |
+        +-------+----------------+
+        | x     | x              |
+        +-------+----------------+
+        | D     | diamond        |
+        +-------+----------------+
+        | d     | thin diamond   |
+        +-------+----------------+
+        | _     | hline          |
+        +-------+----------------+
+        | None  | nothing        |
+        +-------+----------------+
+        | ' '   | nothing        |
+        +-------+----------------+
+        | ''    | nothing        |
+        +-------+----------------+
 
         Marker reference:
         http://matplotlib.org/api/markers_api.html
@@ -467,6 +507,38 @@ def plot_cli(
         Comma separated matplotlib style strings per time-series.  Just
         combine codes in 'ColorMarkerLine' order, for example 'r*--' is
         a red dashed line with star marker.
+    bar_hatchstyles
+        [optional, default to "auto", only used if type equal to "bar", "barh",
+        "bar_stacked", and "barh_stacked"]
+
+        If 'auto' will iterate through the available matplotlib hatch types.
+        Otherwise on the command line a comma separated list, or a list of
+        strings if using the Python API.
+
+        +-----------------+-------------------+
+        | bar_hatchstyles | Description       |
+        +=================+===================+
+        | /               | diagonal hatching |
+        +-----------------+-------------------+
+        | ``\``           | back diagonal     |
+        +-----------------+-------------------+
+        | ``|``           | vertical          |
+        +-----------------+-------------------+
+        | -               | horizontal        |
+        +-----------------+-------------------+
+        | +               | crossed           |
+        +-----------------+-------------------+
+        | x               | crossed diagonal  |
+        +-----------------+-------------------+
+        | o               | small circle      |
+        +-----------------+-------------------+
+        | O               | large circle      |
+        +-----------------+-------------------+
+        | .               | dots              |
+        +-----------------+-------------------+
+        | *               | stars             |
+        +-----------------+-------------------+
+
     logx
         DEPRECATED: use '--xaxis="log"' instead.
     logy
@@ -613,6 +685,7 @@ def plot_cli(
         colors=colors,
         linestyles=linestyles,
         markerstyles=markerstyles,
+        bar_hatchstyles=bar_hatchstyles,
         style=style,
         logx=logx,
         logy=logy,
@@ -694,8 +767,9 @@ def plot_cli(
     sharex=[bool, ["domain", [True, False]], 1],
     sharey=[bool, ["domain", [True, False]], 1],
     colors=[str, ["pass", []], None],
-    linestyles=[str, ["pass", []], None],
-    markerstyles=[str, ["pass", []], None],
+    linestyles=[str, ["domain", ["auto", None, "", " ", "  "] + LINE_LIST], None],
+    markerstyles=[str, ["domain", ["auto", None, "", " ", "  "] + MARKER_LIST], None],
+    bar_hatchstyles=[str, ["domain", ["auto", None, "", " ", "  "] + HATCH_LIST], None],
     style=[str, ["pass", []], None],
     xlim=[float, ["pass", []], 2],
     ylim=[float, ["pass", []], 2],
@@ -746,6 +820,7 @@ def plot(
     colors="auto",
     linestyles="auto",
     markerstyles=" ",
+    bar_hatchstyles="auto",
     style="auto",
     logx=False,
     logy=False,
@@ -855,18 +930,22 @@ Each name in legend_names must be unique.
             )
         if len(tsd.columns) == len(lnames):
             renamedict = dict(list(zip(tsd.columns, lnames)))
-        elif type == "xy" and len(tsd.columns) // 2 == len(lnames):
+        elif type in ["xy", "double_mass"] and (
+            len(tsd.columns) // 2 == len(lnames) or len(tsd.columns) == 1
+        ):
             renamedict = dict(list(zip(tsd.columns[2::2], lnames[1:])))
             renamedict[tsd.columns[1]] = lnames[0]
         else:
             raise ValueError(
                 tsutils.error_wrapper(
                     """
-For 'legend_names' you must have the same number of comma
-separated names as columns in the input data.  The input
-data has {0} where the number of 'legend_names' is {1}.
+For 'legend_names' and most plot types you must have the same number of comma
+separated names as columns in the input data.  The input data has {0} where the
+number of 'legend_names' is {1}.
 
-If 'xy' type you need to have legend names as x,y1,y2,y3,...
+If `type` is 'xy' or 'double_mass' you need to have legend names as
+l1,l2,l3,...  where l1 is the legend for x1,y1, l2 is the legend for x2,y2,
+...etc.
 """.format(
                         len(tsd.columns), len(lnames)
                     )
@@ -886,6 +965,11 @@ If 'xy' type you need to have legend names as x,y1,y2,y3,...
     else:
         linestyles = tsutils.make_list(linestyles)
 
+    if bar_hatchstyles == "auto":
+        bar_hatchstyles = HATCH_LIST
+    else:
+        bar_hatchstyles = tsutils.make_list(bar_hatchstyles)
+
     if markerstyles == "auto":
         markerstyles = MARKER_LIST
     else:
@@ -929,7 +1013,7 @@ but you have {2} time-series.
     if linestyles is None:
         linestyles = [" "]
     else:
-        linestyles = [" " if i == "  " else i for i in linestyles]
+        linestyles = [" " if i in ["  ", None] else i for i in linestyles]
     markerstyles = [" " if i is None else i for i in markerstyles]
 
     icolors = itertools.cycle(colors)
@@ -946,6 +1030,9 @@ but you have {2} time-series.
     imarkerstyles = itertools.cycle(markerstyles)
     ilinestyles = itertools.cycle(linestyles)
 
+    # Only for bar, barh, bar_stacked, and barh_stacked.
+    ibar_hatchstyles = itertools.cycle(bar_hatchstyles)
+
     if (
         logx is True
         or logy is True
@@ -995,13 +1082,13 @@ but you have {2} time-series.
         if logy is True:
             logy = False
             warnings.warn(
-                """
-*
-*   The --type={1} cannot also have the yaxis set to {0}.
-*   The {0} setting for yaxis is ignored.
-*
+                tsutils.error_wrapper(
+                    """
+The --type={1} cannot also have the yaxis set to {0}.
+The {0} setting for yaxis is ignored.
 """.format(
-                    yaxis, type
+                        yaxis, type
+                    )
                 )
             )
 
@@ -1009,22 +1096,33 @@ but you have {2} time-series.
     ylim = _know_your_limits(ylim, axis=yaxis)
 
     figsize = tsutils.make_list(figsize, n=2)
+    _, ax = plt.subplots(figsize=figsize)
 
     if not isinstance(tsd.index, pd.DatetimeIndex):
-        tsd.insert(0, tsd.index.name, tsd.index)
-
-    if type in ["xy", "double_mass"]:
-        if tsd.shape[1] % 2 != 0:
-            raise AttributeError(
-                """
-*
-*   The 'xy' and 'double_mass' types must have an even number of columns
-*   arranged as x,y pairs.  You supplied {0} columns.
-*
-""".format(
-                    tsd.shape[1]
+        if type == "time":
+            raise ValueError(
+                tsutils.error_wrapper(
+                    """
+The index is not a datetime index and cannot be plotted as a time-series.
+Instead of `type="time"` you might want `type="xy"` or change the index to
+a datetime index.
+"""
                 )
             )
+
+    if type in ["xy", "double_mass"]:
+        if tsd.shape[1] > 1:
+            if tsd.shape[1] % 2 != 0:
+                raise AttributeError(
+                    tsutils.error_wrapper(
+                        """
+The 'xy' and 'double_mass' types must have an even number of columns arranged
+as x,y pairs or an x-index and one y data column.  You supplied {0} columns.
+""".format(
+                            tsd.shape[1]
+                        )
+                    )
+                )
         colcnt = tsd.shape[1] // 2
     elif type in [
         "norm_xaxis",
@@ -1036,7 +1134,6 @@ but you have {2} time-series.
     ]:
         colcnt = tsd.shape[1]
 
-    _, ax = plt.subplots(figsize=figsize)
     if type in [
         "xy",
         "double_mass",
@@ -1046,7 +1143,6 @@ but you have {2} time-series.
         "lognorm_yaxis",
         "weibull_xaxis",
         "weibull_yaxis",
-        "heatmap",
     ]:
         plotdict = {
             (False, True): ax.semilogy,
@@ -1113,9 +1209,15 @@ but you have {2} time-series.
         # Boy I do not like matplotlib.
 
         for colindex in range(colcnt):
-            ndf = tsd.iloc[:, colindex * 2 : colindex * 2 + 2]
+            if colcnt == 0:
+                ndf = tsd.reset_index()
+            else:
+                ndf = tsd.iloc[:, colindex * 2 : colindex * 2 + 2]
+
             if type == "double_mass":
-                ndf = ndf.dropna().cumsum()
+                ndf = ndf.dropna()
+                ndf.iloc[:, 0] = ndf.iloc[:, 0].cumsum()
+                ndf.iloc[:, 1] = ndf.iloc[:, 1].cumsum()
             oxdata = np.array(ndf.iloc[:, 0])
             oydata = np.array(ndf.iloc[:, 1])
 
@@ -1131,6 +1233,7 @@ but you have {2} time-series.
 
         ax.set_xlim(xlim)
         ax.set_ylim(ylim)
+
         if legend is True:
             ax.legend(loc="best")
 
@@ -1217,8 +1320,7 @@ but you have {2} time-series.
             ax.legend(loc="best")
 
     elif type in ["kde", "probability_density"]:
-        ax = tsd.plot(
-            kind="kde",
+        ax = tsd.plot.kde(
             legend=legend,
             subplots=subplots,
             sharex=sharex,
@@ -1271,15 +1373,20 @@ but you have {2} time-series.
         xtitle = xtitle or "Time"
         ylimits = ax1.get_ylim()
         ny = np.linspace(ylimits[0], ylimits[1], 1000)
+
+        # reset to beginning of iterator
+        icolors = itertools.cycle(colors)
+        imarkerstyles = itertools.cycle(markerstyles)
+        ilinestyles = itertools.cycle(linestyles)
         for col in range(len(tsd.columns)):
             xvals = tsd.iloc[:, col].dropna().values
             pdf = gaussian_kde(xvals)
             ax0.plot(
                 pdf(ny),
                 ny,
-                linestyle=style[col][2:],
-                color=style[col][0],
-                marker=style[col][1],
+                linestyle=next(ilinestyles),
+                color=next(icolors),
+                marker=next(imarkerstyles),
                 label=tsd.columns[col],
                 drawstyle=drawstyle,
             )
@@ -1363,21 +1470,26 @@ The "heatmap" plot type can only work with daily time series.
         kind = "bar"
         if type[:4] == "barh":
             kind = "barh"
-        ax = tsd.plot(
+        color = [next(icolors) for i in range(len(tsd.columns))]
+        tsd.plot(
+            ax=ax,
             kind=kind,
             legend=legend,
             stacked=stacked,
-            style=style,
             logx=logx,
             logy=logy,
             xlim=xlim,
             ylim=ylim,
             figsize=figsize,
+            linestyle=None,
+            color=color,
         )
-        for index, line in enumerate(ax.lines):
-            plt.setp(line, color=style[index][0])
-            plt.setp(line, marker=style[index][1])
-            plt.setp(line, linestyle=style[index][2:])
+
+        hatches = [next(ibar_hatchstyles) for i in range(len(tsd.columns))]
+        hatches = "".join(h * len(tsd.index) for h in hatches)
+        for patch, hatch in zip(ax.patches, hatches):
+            patch.set_hatch(hatch)
+
         freq = tsutils.asbestfreq(tsd, force_freq=force_freq).index.freqstr
         if freq is not None:
             if "A" in freq:
@@ -1405,7 +1517,8 @@ The "heatmap" plot type can only work with daily time series.
         if legend is True:
             plt.legend(loc="best")
     elif type == "histogram":
-        tsd.hist(figsize=figsize)
+        tsd.hist(figsize=figsize, sharey=sharey, sharex=sharex)
+
     if xy_match_line:
         if isinstance(xy_match_line, str):
             xymsty = xy_match_line
