@@ -38,8 +38,125 @@ def error_wrapper(estr):
     return "\n".join(nestr)
 
 
+_CODES = {}
+_CODES["SUB_D"]: {
+    "N": "Nanoseconds",
+    "U": "microseconds",
+    "L": "miLliseconds",
+    "S": "Secondly",
+    "T": "minuTely",
+    "H": "Hourly",
+}
+_CODES["DAILY"]: {
+    "D": "calendar Day",
+    "B": "Business day",
+    "C": "Custom business day (experimental)",
+}
+_CODES["WEEKLY"]: {
+    "W": "Weekly",
+    "W-SUN": "Weekly frequency (SUNdays)",
+    "W-MON": "Weekly frequency (MONdays)",
+    "W-TUE": "Weekly frequency (TUEsdays)",
+    "W-WED": "Weekly frequency (WEDnesdays)",
+    "W-THU": "Weekly frequency (THUrsdays)",
+    "W-FRI": "Weekly frequency (FRIdays)",
+    "W-SAT": "Weekly frequency (SATurdays)",
+}
+_CODES["MONTH"]: {
+    "M": "Month end",
+    "MS": "Month Start",
+    "BM": "Business Month end",
+    "BMS": "Business Month Start",
+    "CBM": "Custom Business Month end",
+    "CBMS": "Custom Business Month Start",
+}
+_CODES["QUARTERLY"]: {
+    "Q": "Quarter end",
+    "Q-JAN": "Quarterly, quarter ends end of JANuary",
+    "Q-FEB": "Quarterly, quarter ends end of FEBruary",
+    "Q-MAR": "Quarterly, quarter ends end of MARch",
+    "Q-APR": "Quarterly, quarter ends end of APRil",
+    "Q-MAY": "Quarterly, quarter ends end of MAY",
+    "Q-JUN": "Quarterly, quarter ends end of JUNe",
+    "Q-JUL": "Quarterly, quarter ends end of JULy",
+    "Q-AUG": "Quarterly, quarter ends end of AUGust",
+    "Q-SEP": "Quarterly, quarter ends end of SEPtember",
+    "Q-OCT": "Quarterly, quarter ends end of OCTober",
+    "Q-NOV": "Quarterly, quarter ends end of NOVember",
+    "Q-DEC": "Quarterly, quarter ends end of DECember",
+    "QS": "Quarter Start",
+    "QS-JAN": "Quarterly, quarter Starts end of JANuary",
+    "QS-FEB": "Quarterly, quarter Starts end of FEBruary",
+    "QS-MAR": "Quarterly, quarter Starts end of MARch",
+    "QS-APR": "Quarterly, quarter Starts end of APRil",
+    "QS-MAY": "Quarterly, quarter Starts end of MAY",
+    "QS-JUN": "Quarterly, quarter Starts end of JUNe",
+    "QS-JUL": "Quarterly, quarter Starts end of JULy",
+    "QS-AUG": "Quarterly, quarter Starts end of AUGust",
+    "QS-SEP": "Quarterly, quarter Starts end of SEPtember",
+    "QS-OCT": "Quarterly, quarter Starts end of OCTober",
+    "QS-NOV": "Quarterly, quarter Starts end of NOVember",
+    "QS-DEC": "Quarterly, quarter Starts end of DECember",
+    "BQ": "Business Quarter end",
+    "BQS": "Business Quarter Start",
+}
+_CODES["ANNUAL"]: {
+    "A": "Annual end",
+    "A-JAN": "Annual, year ends end of JANuary",
+    "A-FEB": "Annual, year ends end of FEBruary",
+    "A-MAR": "Annual, year ends end of MARch",
+    "A-APR": "Annual, year ends end of APRil",
+    "A-MAY": "Annual, year ends end of MAY",
+    "A-JUN": "Annual, year ends end of JUNe",
+    "A-JUL": "Annual, year ends end of JULy",
+    "A-AUG": "Annual, year ends end of AUGust",
+    "A-SEP": "Annual, year ends end of SEPtember",
+    "A-OCT": "Annual, year ends end of OCTober",
+    "A-NOV": "Annual, year ends end of NOVember",
+    "A-DEC": "Annual, year ends end of DECember",
+    "AS": "Annual Start",
+    "AS-JAN": "Annual, year Starts end of JANuary",
+    "AS-FEB": "Annual, year Starts end of FEBruary",
+    "AS-MAR": "Annual, year Starts end of MARch",
+    "AS-APR": "Annual, year Starts end of APRil",
+    "AS-MAY": "Annual, year Starts end of MAY",
+    "AS-JUN": "Annual, year Starts end of JUNe",
+    "AS-JUL": "Annual, year Starts end of JULy",
+    "AS-AUG": "Annual, year Starts end of AUGust",
+    "AS-SEP": "Annual, year Starts end of SEPtember",
+    "AS-OCT": "Annual, year Starts end of OCTober",
+    "AS-NOV": "Annual, year Starts end of NOVember",
+    "AS-DEC": "Annual, year Starts end of DECember",
+    "BA": "Business Annual end",
+    "BA-JAN": "Business Annual, business year ends end of JANuary",
+    "BA-FEB": "Business Annual, business year ends end of FEBruary",
+    "BA-MAR": "Business Annual, business year ends end of MARch",
+    "BA-APR": "Business Annual, business year ends end of APRil",
+    "BA-MAY": "Business Annual, business year ends end of MAY",
+    "BA-JUN": "Business Annual, business year ends end of JUNe",
+    "BA-JUL": "Business Annual, business year ends end of JULy",
+    "BA-AUG": "Business Annual, business year ends end of AUGust",
+    "BA-SEP": "Business Annual, business year ends end of SEPtember",
+    "BA-OCT": "Business Annual, business year ends end of OCTober",
+    "BA-NOV": "Business Annual, business year ends end of NOVember",
+    "BA-DEC": "Business Annual, business year ends end of DECember",
+    "BAS": "Business Annual Start",
+    "BS-JAN": "Business Annual Start, business year starts end of JANuary",
+    "BS-FEB": "Business Annual Start, business year starts end of FEBruary",
+    "BS-MAR": "Business Annual Start, business year starts end of MARch",
+    "BS-APR": "Business Annual Start, business year starts end of APRil",
+    "BS-MAY": "Business Annual Start, business year starts end of MAY",
+    "BS-JUN": "Business Annual Start, business year starts end of JUNe",
+    "BS-JUL": "Business Annual Start, business year starts end of JULy",
+    "BS-AUG": "Business Annual Start, business year starts end of AUGust",
+    "BS-SEP": "Business Annual Start, business year starts end of SEPtember",
+    "BS-OCT": "Business Annual Start, business year starts end of OCTober",
+    "BS-NOV": "Business Annual Start, business year starts end of NOVember",
+    "BS-DEC": "Business Annual Start, business year starts end of DECember",
+}
+
 docstrings = {
-    "target_units": r"""target_units
+    "target_units": r"""target_units: str
         [optional, default is None, transformation]
 
         The purpose of this option is to specify target units for unit
@@ -54,7 +171,7 @@ docstrings = {
 
         This option will also add the 'target_units' string to the
         column names.""",
-    "source_units": r"""source_units
+    "source_units": r"""source_units: str
         [optional, default is None, transformation]
 
         If unit is specified for the column as the second field of a ':'
@@ -63,7 +180,7 @@ docstrings = {
 
         Any unit string compatible with the 'pint' library can be
         used.""",
-    "names": r"""names
+    "names": r"""names: str
         [optional, default is None, transformation]
 
         If None, the column names are taken from the first row after
@@ -185,56 +302,62 @@ docstrings = {
         This is if you want a different header than is the default for
         this output table.  Pass a list of strings for each column in
         the table.""",
-    "pandas_offset_codes": r"""+-------+------------------------------------+
+    "pandas_offset_codes": r"""+-------+---------------+
+        | Alias | Description   |
+        +=======+===============+
+        | N     | Nanoseconds   |
+        +-------+---------------+
+        | U     | microseconds  |
+        +-------+---------------+
+        | L     | milliseconds  |
+        +-------+---------------+
+        | S     | Secondly      |
+        +-------+---------------+
+        | T     | Minutely      |
+        +-------+---------------+
+        | H     | Hourly        |
+        +-------+---------------+
+        | D     | calendar Day  |
+        +-------+---------------+
+        | W     | Weekly        |
+        +-------+---------------+
+        | M     | Month end     |
+        +-------+---------------+
+        | MS    | Month Start   |
+        +-------+---------------+
+        | Q     | Quarter end   |
+        +-------+---------------+
+        | QS    | Quarter Start |
+        +-------+---------------+
+        | A     | Annual end    |
+        +-------+---------------+
+        | AS    | Annual Start  |
+        +-------+---------------+
+
+        Business offset codes.
+
+        +-------+------------------------------------+
         | Alias | Description                        |
         +=======+====================================+
         | B     | Business day                       |
         +-------+------------------------------------+
-        | C     | Custom business day (experimental) |
-        +-------+------------------------------------+
-        | D     | calendar Day                       |
-        +-------+------------------------------------+
-        | W     | Weekly                             |
-        +-------+------------------------------------+
-        | M     | Month end                          |
-        +-------+------------------------------------+
         | BM    | Business Month end                 |
         +-------+------------------------------------+
-        | CBM   | Custom Business Month end          |
-        +-------+------------------------------------+
-        | MS    | Month Start                        |
-        +-------+------------------------------------+
         | BMS   | Business Month Start               |
         +-------+------------------------------------+
-        | CBMS  | Custom Business Month Start        |
-        +-------+------------------------------------+
-        | Q     | Quarter end                        |
-        +-------+------------------------------------+
         | BQ    | Business Quarter end               |
         +-------+------------------------------------+
-        | QS    | Quarter Start                      |
-        +-------+------------------------------------+
         | BQS   | Business Quarter Start             |
         +-------+------------------------------------+
-        | A     | Annual end                         |
-        +-------+------------------------------------+
         | BA    | Business Annual end                |
         +-------+------------------------------------+
-        | AS    | Annual Start                       |
-        +-------+------------------------------------+
         | BAS   | Business Annual Start              |
         +-------+------------------------------------+
-        | H     | Hourly                             |
+        | C     | Custom business day (experimental) |
         +-------+------------------------------------+
-        | T     | Minutely                           |
+        | CBM   | Custom Business Month end          |
         +-------+------------------------------------+
-        | S     | Secondly                           |
-        +-------+------------------------------------+
-        | L     | milliseconds                       |
-        +-------+------------------------------------+
-        | U     | microseconds                       |
-        +-------+------------------------------------+
-        | N     | Nanoseconds                        |
+        | CBMS  | Custom Business Month Start        |
         +-------+------------------------------------+
 
         Weekly has the following anchored frequencies:
@@ -361,8 +484,28 @@ docstrings = {
         careful with this option.  In general, letting the algorithm
         determine the frequency should always work, but this option will
         override.  Use PANDAS offset codes.""",
+    "output_names": r"""output_names: str
+        [optional, output_format]
+
+        The tstoolbox will change the names of the output columns to include
+        some record of the operations used on each column.  The `output_names`
+        will override that feature.  Must be a list or tuple equal to the
+        number of columns in the output data.""",
 }
 
+# Decided this was inelegant, but left here in case I figure out what I want
+# and how I want it.
+# ntables = {}
+# for key in ["SUB_D", "DAILY", "WEEKLY", "QUATERLY", "ANNUAL"]:
+#     ntables[key] = tb(_CODES[key].items(), tablefmt="grid", headers=["Alias", "Description"],)
+#     ntables[key] = "        ".join(ntables[key].splitlines(True))
+# codes_table = f"""{ntables["SUB_D"]}
+#
+#     {ntables["DAILY"]}
+#     """
+#
+# docstrings["pandas_offset_codes"] = codes_table
+
 
 def stride_and_unit(sunit):
     """Split a stride/unit combination into component parts."""
@@ -442,6 +585,8 @@ def doc(fdict, **kwargs):
 
     def f(fn):
         fn.__doc__ = fn.__doc__.format(**fdict)
+        # kwargs is currently always empty.
+        # Could remove, but keeping in case useful in future.
         for attr in kwargs:
             setattr(fn, attr, kwargs[attr])
         return fn
@@ -449,6 +594,15 @@ def doc(fdict, **kwargs):
     return f
 
 
+def convert_keyword_to_postional(keyword_name, *args, **kwargs):
+    """ When complete will convert keyword_name from **kwargs to end of *args. """
+
+    def f(fn):
+        return fn
+
+    return f
+
+
 def parsedate(dstr, strftime=None, settings=None):
     """Use dateparser to parse a wide variety of dates.
 
@@ -661,6 +815,21 @@ The list {0} for "{2}" should have {1} members according to function requirement
     return ret
 
 
+def make_iloc(columns, col_list):
+    """Imitates the .ix option with subtracting one to convert."""
+
+    # ["1", "Value2"]    ->    [0, "Value2"]
+    # [1, 2, 3]          ->    [0, 1, 2]
+    col_list = make_list(col_list)
+    ntestc = []
+    for i in col_list:
+        try:
+            ntestc.append(int(i) - 1)
+        except ValueError:
+            ntestc.append(columns.index(i))
+    return ntestc
+
+
 # NOT SET YET...
 #
 # Take `air_pressure` from df.loc[:, 1]
@@ -1091,8 +1260,8 @@ The name {0} isn't in the list of column names
             raise ValueError(
                 error_wrapper(
                     """
-The requested column index {0} must be greater than or equal to 0.
-First data column is index 1, index is column 0.
+The requested column "{0}" must be greater than or equal to 0.
+First data column is 1, index is column 0.
 """.format(
                         i
                     )
@@ -1121,9 +1290,9 @@ number of columns {1}.
     for index, col in enumerate(ncolumns):
         if col == -1:
             # Use the -1 marker to indicate index
-            jtsd = pd.DataFrame(tsd.index)
+            jtsd = pd.DataFrame(tsd.index, index=tsd.index)
         else:
-            jtsd = pd.DataFrame(tsd[tsd.columns[col]])
+            jtsd = pd.DataFrame(tsd[tsd.columns[col]], index=tsd.index)
 
         newtsd = newtsd.join(jtsd, lsuffix="_{0}".format(index), how="outer")
     return newtsd
@@ -1361,9 +1530,11 @@ def print_input(
     )
 
 
-def return_input(iftrue, intds, output, suffix="", reverse_index=False):
+def return_input(
+    iftrue, intds, output, suffix="", reverse_index=False, output_names=[]
+):
     """Print the input time series also."""
-    output.columns = [renamer(i, suffix) for i in output.columns]
+    output.columns = output_names or [renamer(i, suffix) for i in output.columns]
     if iftrue:
         return intds.join(output, lsuffix="_1", rsuffix="_2", how="outer")
     if reverse_index is True:
